
/*--Add "Id" to identify subGroup headings from values                      --*/
data forest;
  input Id SubGroup $3-27 Count Percent Mean  Low  High  PCIGroup Group PValue;
  label PCIGroup='PCI Group' PValue='P Value';
  indentWt=1;
  ObsId=_n_; 
  datalines;
1 Overall                  2166  100  1.3   0.9   1.5  17.2  15.6  .
1 Age                      .     .    .     .     .    .     .     0.05
2 <= 65 Yr                 1534   71  1.5   1.05  1.9  17.0  13.2   .
2 > 65 Yr                   632   29  0.8   0.6   1.25 17.8  21.3   .
1 Sex                      .     .    .     .     .    .     .     0.13
2 Male                     1690   78  1.5   1.05  1.9  16.8  13.5   .
2 Female                    476   22  0.8   0.6   1.3  18.3  22.9   . 
1 Race or ethnic group     .     .    .     .     .    .     .     0.52
2 Nonwhite                  428   20  1.05  0.6   1.8  18.8  17.8   .
2 White                    1738   80  1.2   0.85  1.6  16.7  15.0   . 
1 From MI to Randomization .     .    .     .     .    .     .     0.81
2 <= 7 days                 963   44  1.2   0.8   1.5  18.9  18.6   .
2 > 7 days                 1203   56  1.15  0.75  1.5  15.9  12.9   .
1 Infract-related artery   .     .    .     .     .    .     .     0.38
2 LAD                       781   36  1.4   0.9   1.9  20.1  16.2   .
2 Other                    1385   64  1.1   0.8   1.4  15.6  15.3   . 
1 Ejection Fraction        .     .    .     .     .    .     .     0.48
2 < 50%                    1151   54  1.2   0.8   1.5  22.6  20.4   .
2 >= 50%                    999   46  0.9   0.6   1.4  10.7  11.1   . 
1 Diabetes                 .     .    .     .     .    .     .     0.41
2 Yes                       446   21  1.4   0.9   2.0  29.3  23.3   .
2 No                       1720   79  1.1   0.8   1.5  14.4  13.5   . 
1 Killip class             .     .    .     .     .    .     .     0.39
2 I                        1740   81  1.2   0.8   1.6  15.2  13.1   .
2 II-IV                     413   19  0.95  0.6   1.5  25.3  26.9   . 
;
run;

data forest2;
  set forest;
  if count ne . then CountPct=put(count, 4.0) || "(" || put(percent, 3.0) || ")";
  if mod(obsId-1, 6) in (1, 2, 3) then ref=obsId;
    /* Reduce indent for subGroup heading */
  if id=1 then indentWt=0;
run;

/**
 * Forest plot template using a one-column outermost lattice layout.
 * The header uses a nested lattice layout in the top side bar. 
 * An overlay layout makes up the main area below it.
 * The left side table of values are axis tables placed inside
 * inner margin of the overlay layout.
 */
proc template;
  define statgraph forestAxisTable;
    dynamic  _bandColor _headerColor;
    begingraph;

      discreteattrmap name='text' / trimleading=true;
        value '1' / textAttrs=(size=7 weight=bold); 
        value '2' / textAttrs=(color=gray size=6 weight=normal); 
      enddiscreteattrmap;

      discreteattrvar attrvar=txtDAV var=id attrmap='text';

      layout lattice / columns=1;

         /*--Column headers--*/
        sidebar / align=top;
          layout lattice / columns=2 columnweights=(0.38 0.62)
                            backgroundcolor=_headerColor opaque=true;
              entry textAttrs=(size=8 weight=bold) hAlign=left "Subgroup"
                                          hAlign=right " No. of Patients (%)";
              entry textAttrs=(size=8 weight=bold) "Hazard Ratio";
          endLayout;
        endsidebar;

        /* Single cell with inner margins for left and right tables */
        layout overlay / xAxisOpts=(display=(line ticks tickvalues) 
                            tickValueAttrs=(size=6 weight=bold)
                            labelAttrs=(size=6 weight=bold)
                            linearOpts=(tickValuePriority=true 
                                        tickValueList=(0.0 0.5 1.0 1.5 2.0))
                              offsetMin=0.1
                            )
                      yAxisOpts=(reverse=true display=none offsetMin=0) wallDisplay=none;

          /* Left-side table */
          innerMargin / align=left;
            axisTable y=obsId value=subGroup / textGroup=txtDAV
                          indentWeight=indentwt display=(values);
            axisTable y=obsId value=CountPct /  display=(values);
          endInnerMargin;

          /* Odds Ratio plot */
          referenceLine y=ref / lineAttrs=(thickness=15 color=_bandColor);
          scatterPlot y=obsId x=mean / markerAttrs=(symbol=squareFilled)
                          xErrorLower=low xErrorUpper=high errorBarCapShape=none;
          referenceLine x=1;

        endLayout; /* overlay */

      endLayout; /* lattice */
    endgraph;
  end;
run;

ods _all_ close;
ods listing image_dpi=300;
ods graphics / reset width=6in height=4.5in imagename='SAS94M2_ForestPlot_GTL';

proc sgrender data=forest2 template=forestAxisTable;
  dynamic _bandColor='cxf0f0f0' _headerColor='cxd0d0d0';
run;

ods _all_ close;
