/** ***
 * Forest plot for SAS 9.4 TS1M3 using AxisTable, axisLineExtent=
 */

/*--"Id" differentiates subgroup headings from values                    --*/
data forest;
  input Id SubGroup $3-27 Count Percent Mean  Low  High  PCIGroup Group PValue;
  label PCIGroup='PCI Group' Group='Medical Therapy Group' PValue='P Value';
  datalines;
1 Overall                  2166  100  1.3   0.9   1.5  17.2  15.6  .
1 Age                      .     .    .     .     .    .     .     0.05
2 <= 65 Yr                 1534   71  1.5   1.05  1.9  17.0  13.2   .
2 > 65 Yr                   632   29  0.8   0.6   1.25 17.8  21.3   .
1 Sex                      .     .    .     .     .    .     .     0.13
2 Male                     1690   78  1.5   1.05  1.9  16.8  13.5   .
2 Female                    476   22  0.8   0.6   1.3  18.3  22.9   . 
1 Race or ethnic group     .     .    .     .     .    .     .     0.52
2 Nonwhite                  428   20  1.05  0.6   1.8  18.8  17.8   .
2 White                    1738   80  1.2   0.85  1.6  16.7  15.0   . 
1 From MI to Randomization .     .    .     .     .    .     .     0.81
2 <= 7 days                 963   44  1.2   0.8   1.5  18.9  18.6   .
2 > 7 days                 1203   56  1.15  0.75  1.5  15.9  12.9   .
1 Infract-related artery   .     .    .     .     .    .     .     0.38
2 LAD                       781   36  1.4   0.9   1.9  20.1  16.2   .
2 Other                    1385   64  1.1   0.8   1.4  15.6  15.3   . 
1 Ejection Fraction        .     .    .     .     .    .     .     0.48
2 < 50%                    1151   54  1.2   0.8   1.5  22.6  20.4   .
2 >= 50%                    999   46  0.9   0.6   1.4  10.7  11.1   . 
1 Diabetes                 .     .    .     .     .    .     .     0.41
2 Yes                       446   21  1.4   0.9   2.0  29.3  23.3   .
2 No                       1720   79  1.1   0.8   1.5  14.4  13.5   . 
1 Killip class             .     .    .     .     .    .     .     0.39
2 I                        1740   81  1.2   0.8   1.6  15.2  13.1   .
2 II-IV                     413   19  0.95  0.6   1.5  25.3  26.9   . 
;
run;

/* Add variables for reference band, count(%) and indentation */
data forest2;
  set forest;

  ObsId=_n_; 
  if count ne . then CountPct=put(count, 4.0) || "(" || put(percent, 3.0) || ")";
  if mod(_N_-1, 6) in (1, 2, 3) then ref=obsId;

  indentWt=1;
  if id=1 then indentWt=0; /* Reduce indent for subGroup heading */
run;

/* Format values into unicode escapes for use with labels */
proc format;
  value unicodeLabel
    1 = "(*ESC*){unicode '2190'x} PCI Better -"
    2 = "- Therapy Better (*ESC*){unicode '2192'x}"
  ;
run;

data bottomLabel;
  attrib textLeft format=unicodeLabel.
          textRight format=unicodeLabel.;
  labXLeft = 0.9; labXRight= 1.1; labY = 27; textLeft = 1; textRight = 2;
  output;
run;

data forestFinal;
  merge forest2 bottomLabel;
run;

/**
 * Forest plot template using a one-column outermost lattice layout.
 * The header uses a nested lattice layout in the top side bar. 
 * An overlay layout makes up the main area below it.
 * The left and right side table of values are axis tables placed inside
 * inner margins of the overlay layout.
 */
proc template;
  define statgraph forestAxisTable;
    dynamic  _bandcolor _headercolor _subGroupcolor;
    begingraph;

      discreteattrmap name='text' / ignorecase=true trimleading=true;
        value '1' / textAttrs=(size=7 weight=bold); 
        value '2' / textAttrs=(color=gray size=6 weight=normal); 
      enddiscreteattrmap;

      discreteattrvar attrvar=grp var=id attrmap='text';

      layout lattice / columns=1;

         /*--Column headers--*/
        sidebar / align=top;
          layout lattice / columns=3 columnweights=(0.3 0.42 0.28)
              backgroundcolor=_headercolor opaque=true;
              entry textAttrs=(size=8 weight=bold)
                                      halign=left "Subgroup"
                                      hAlign=right " No.of Patients (%)";
              entry textAttrs=(size=8 weight=bold) "Hazard Ratio";
              entry textAttrs=(size=8 weight=bold)
                                halign=left "4-Yr Cumulative Event Rate"
                                hAlign=right "P Value";
          endLayout;
        endsidebar;

        /* Single cell with inner margins for left and right tables */
        layout overlay / xAxisOpts=(display=(line ticks tickvalues) 
                            tickValueAttrs=(size=6 weight=bold)
                            labelAttrs=(size=6 weight=bold)
                            linearOpts=(tickValuePriority=true 
                                        tickValueList=(0.0 0.5 1.0 1.5 2.0 2.5))
                            lineExtent=data)
                      yAxisOpts=(reverse=true display=none offsetMin=0) wallDisplay=none;

          /* Left-side table */
          innerMargin / align=left;
            axisTable y=obsId value=subGroup / textGroup=grp
                          indentWeight=indentwt display=(values);
            axisTable y=obsId value=CountPct /  display=(values);
          endInnerMargin;

          /* Odds Ratio plot */
          referenceLine y=ref / lineAttrs=(thickness=12 color=_bandcolor);
          highLowPlot y=obsId low=low high=high; 
          scatterPlot y=obsId x=mean / markerAttrs=(symbol=squareFilled);
          referenceLine x=1;

          /* better indicator */
          textPlot y=labY x=labXLeft text=textLeft / strip=true
                  position=left contributeOffsets=(yMin);
          textPlot y=labY x=labXRight text=textRight / strip=true
                  position=right contributeOffsets=(yMin);

          /* Right-side table */
          innerMargin / align=right;
%let commonRightTableOpts = display=(values label) showMissing=false
                              labelAttrs=(size=6 weight=bold) ;
            axisTable y=obsId value=PCIGroup / &commonRightTableOpts;
            axisTable y=obsId value=group /  &commonRightTableOpts;
            axisTable y=obsId value=pvalue / &commonRightTableOpts
                                           pad=(left=9%) display=(values);
          endInnerMargin;
        endLayout; /* overlay */

      endLayout; /* lattice */
      entryfootnote hAlign=left textAttrs=(size=7) 
               'The p-value is from the test statistic for testing the '
               'interaction between the treatment and any subgroup variable';
    endgraph;
  end;
run;

ods _all_ close;
ods listing image_dpi=300;
ods graphics / reset width=7.5in height=4.5in imagename='SAS94M3_ForestPlot';

proc sgrender data=forestFinal template=forestAxisTable;
  dynamic _bandcolor='cxf0f0f0' _headercolor='cxd0d0d0';
run;
ods _all_ close;
